package cz.muni.fi.pa165.books;

import cz.muni.fi.pa165.books.validation.BookConstraint;
import cz.muni.fi.pa165.books.validation.Censorship;
import org.hibernate.validator.constraints.NotBlank;

import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Pattern;
import javax.validation.constraints.Size;

/**
 * Book data transfer object.
 *
 * @author Martin Kuba makub@ics.muni.cz
 */
@BookConstraint
public class Book implements Cloneable {

    public enum Color { BLUE, GREEN, RED, WHITE, ORANGE, OTHER }

    private Long id;
    @NotNull
    @Pattern(regexp = "\\p{javaUpperCase}.*")
    @Size(min = 1, max = 30)
    private String author; //field-level validation

    @NotBlank
    @Size(min = 1, max = 30)
    @Censorship({"bomb","missile","plutonium"}) //validation with custom constraint
    private String title;

    private boolean paperback;

    private Color color;

    @Min(1)
    private int price;


    public Book() {
    }

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getTitle() {
        return title;
    }

    public void setTitle(String title) {
        this.title = title;
    }

    public String getAuthor() {
        return author;
    }

    public void setAuthor(String author) {
        this.author = author;
    }

    public boolean isPaperback() {
        return paperback;
    }

    public void setPaperback(boolean paperback) {
        this.paperback = paperback;
    }

    public Color getColor() {
        return color;
    }

    public void setColor(Color color) {
        this.color = color;
    }

    public int getPrice() {
        return price;
    }

    public void setPrice(int price) {
        this.price = price;
    }

    @Override
    public String toString() {
        return "Book{" +
                "id=" + id +
                ", title='" + title + '\'' +
                ", author='" + author + '\'' +
                ", paperback=" + paperback +
                ", color=" + color +
                ", price=" + price +
                '}';
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        Book book = (Book) o;

        if (paperback != book.paperback) return false;
        if (price != book.price) return false;
        if (author != null ? !author.equals(book.author) : book.author != null) return false;
        if (color != book.color) return false;
        if (id != null ? !id.equals(book.id) : book.id != null) return false;
        if (title != null ? !title.equals(book.title) : book.title != null) return false;

        return true;
    }

    @Override
    public int hashCode() {
        int result = id != null ? id.hashCode() : 0;
        result = 31 * result + (title != null ? title.hashCode() : 0);
        result = 31 * result + (author != null ? author.hashCode() : 0);
        result = 31 * result + (paperback ? 1 : 0);
        result = 31 * result + (color != null ? color.hashCode() : 0);
        result = 31 * result + price;
        return result;
    }

    @Override
    public Object clone()  {
        try {
            return super.clone();
        } catch (CloneNotSupportedException e) {
            throw new RuntimeException(e);
        }
    }
}
