package cz.muni.fi.pa165.books.validation;

import javax.validation.Constraint;
import javax.validation.Payload;
import java.lang.annotation.*;

/**
 * Custom validation annotation for excluding strings containing censored words.
 * Uses JSR-303 Java Bean validation.
 * See http://docs.jboss.org/hibernate/stable/validator/reference/en-US/html_single/#validator-customconstraints
 *
 * @author Martin Kuba makub@ics.muni.cz
 */
@Target({ElementType.FIELD, ElementType.ANNOTATION_TYPE})
@Retention(RetentionPolicy.RUNTIME)
@Documented
@Constraint(validatedBy = CensorshipValidator.class) //where the validation work is performed
public @interface Censorship {

    //see http://docs.jboss.org/hibernate/stable/validator/reference/en-US/html_single/#section-message-interpolation
    //text is in src/main/resources/ValidationMessages.properties
    String message() default "{cz.muni.fi.pa165.books.validation.Censorship}";

    Class<?>[] groups() default {};

    Class<? extends Payload>[] payload() default {};

    String[] value();
}
