package cz.muni.fi.pa165.books.validation;

import org.hibernate.validator.constraintvalidation.HibernateConstraintValidatorContext;

import javax.validation.ConstraintValidator;
import javax.validation.ConstraintValidatorContext;

/**
 * JSR-303 "Java Bean validation" validator for @Censorship annotation.
 * Searches given string for forbidden texts.
 *
 * @author Martin Kuba makub@ics.muni.cz
 */
public class CensorshipValidator implements ConstraintValidator<Censorship, String> {

    private String[] censored;

    @Override
    public void initialize(Censorship c) {
        censored = c.value();
    }

    @Override
    public boolean isValid(String value, ConstraintValidatorContext ctx) {
        if (value == null) return true;
        for (String s : censored) {
            if (value.contains(s)) {
                //https://docs.jboss.org/hibernate/validator/5.1/reference/en-US/html/validator-specifics.html#section-hibernateconstraintvalidatorcontext
                ctx.unwrap(HibernateConstraintValidatorContext.class).addExpressionVariable("found", s);
                return false;
            }
        }
        return true;
    }

}
