package cz.muni.fi.pa165.books.jpa;

import cz.muni.fi.pa165.books.Book;
import cz.muni.fi.pa165.books.BookService;
import cz.muni.fi.pa165.books.Customer;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.util.ArrayList;
import java.util.List;

@Component("bookService")
public class BookServiceJPAImpl implements BookService {

    @Autowired
    private BookDAO bookDAO;

    @Autowired
    private CustomerDAO customerDAO;

    @PostConstruct
    public void preloadDB() {
        System.out.println("preloadDB");

        Book b1 = new Book();
        b1.setAuthor("Božena Němcová");
        b1.setTitle("Babička");
        b1.setPaperback(false);
        b1.setColor(Book.Color.ORANGE);
        b1.setPrice(200);
        createBook(b1);
        Book b2 = new Book();
        b2.setAuthor("Karel May");
        b2.setTitle("Vinnetou");
        b2.setPaperback(true);
        b2.setColor(Book.Color.GREEN);
        b2.setPrice(90);
        createBook(b2);

        Customer c1 = new Customer();
        c1.setFirstName("Jan");
        c1.setLastName("Novák");
        c1.setAddress("Severní 13");
        c1.setPhone("+420123456789");
        createCustomer(c1);
    }

    private static Book entity2Book(BookEntity be) {
        Book book = new Book();
        book.setId((long) be.getId());
        book.setAuthor(be.getAuthor());
        book.setTitle(be.getTitle());
        book.setPaperback(be.isPaperback());
        book.setPrice(be.getPrice());
        book.setColor(Book.Color.valueOf(be.getColor()));
        return book;
    }

    private static BookEntity book2Entity(Book b) {
        BookEntity be = new BookEntity();
        be.setId(b.getId() == null ? 0 : b.getId().intValue());
        be.setAuthor(b.getAuthor());
        be.setTitle(b.getTitle());
        be.setPaperback(b.isPaperback());
        be.setPrice(b.getPrice());
        be.setColor(b.getColor().toString());
        return be;
    }

    @Override
    public List<Book> getAllBooks() {
        List<BookEntity> bookEntities = bookDAO.getAllBooks();
        List<Book> books = new ArrayList<>();
        for (BookEntity bookEntity : bookEntities) {
            books.add(entity2Book(bookEntity));
        }
        return books;
    }

    @Override
    public void createBook(Book book) {
        bookDAO.createBook(book2Entity(book));
    }

    @Override
    public Book getBook(Long id) {
        return entity2Book(bookDAO.getBookById(id.intValue()));
    }

    @Override
    public void updateBook(Book book) {
        bookDAO.updateBook(book2Entity(book));
    }

    @Override
    public void deleteBook(Book book) {
        bookDAO.deleteBook(book2Entity(book));
    }

    private static Customer entity2Customer(CustomerEntity ce) {
        Customer c = new Customer();
        c.setId(ce.getId());
        c.setFirstName(ce.getFirstName());
        c.setLastName(ce.getLastName());
        c.setAddress(ce.getAddress());
        c.setPhone(ce.getPhone());
        return c;
    }

    private static CustomerEntity customer2Entity(Customer c) {
        CustomerEntity ce = new CustomerEntity();
        ce.setId(c.getId());
        ce.setFirstName(c.getFirstName());
        ce.setLastName(c.getLastName());
        ce.setAddress(c.getAddress());
        ce.setPhone(c.getPhone());
        return ce;
    }

    @Override
    public List<Customer> getAllCustomers() {
        List<CustomerEntity> CustomerEntities = customerDAO.getAllCustomers();
        List<Customer> customers = new ArrayList<>();
        for (CustomerEntity customerEntity : CustomerEntities) {
            customers.add(entity2Customer(customerEntity));
        }
        return customers;
    }

    @Override
    public void createCustomer(Customer customer) {
        customerDAO.createCustomer(customer2Entity(customer));
    }

    @Override
    public Customer getCustomer(Long id) {
        return entity2Customer(customerDAO.getCustomer(id));
    }

    @Override
    public void updateCustomer(Customer customer) {
        customerDAO.updateCustomer(customer2Entity(customer));
    }

    @Override
    public void deleteCustomer(Customer customer) {
        customerDAO.deleteCustomer(customer2Entity(customer));
    }

}
